function MASS=mass(Xcal,ycal,nLV_max,fold,method,num_bms,flag)
% MASS
% Input: 
%        Xcal: The calibration data matrix of size m x n
%        ycal: The calibration response vector of size m x 1
%        nLV_max: The maximum number of latent variable
%        fold: The group number for cross validation
%        method: Pretreatment method
%        num_bms: The number of binray matrix sampling
%        flag: If flag=0,do not present the plots, if flag=1, present the plots
% Output:
%        MASS: the result of MASS
% Coded by:       
%        Ming Wen


%%        Initial settings
if nargin<7; flag=0;end;
if nargin<6; num_bms=1000;end;
if nargin<5; method=('center');end;
if nargin<4; fold=5;end;
if nargin<3; nLV_max=10;end

if num_bms>=2000 
    ratio=0.05;
else if num_bms<2000 && num_bms>=1000 
        ratio=0.1;
    else if num_bms<1000 && num_bms>=500
            ratio=0.2;
        else if num_bms<500
                fprintf('The number of BMS ought to be larger than 500')
                MASS=[];
                return
            end
        end
    end
end

%%     The main program
% tic
[m,n]=size(Xcal);   %%%%%  %%%%%
num_selected_model=num_bms*ratio; %% the number of selected model


weight_descriptors=ones(n,1)*0.5; %% set the initial weight for variables
W_de=[weight_descriptors zeros(n,200)];


weight_samples=ones(m,1)*0.95; %% set the initial weight for samples
W_sa=[weight_samples zeros(m,200)];

meanRMSECV=zeros(1,200);

for i=1:200
    %    generate the binary matrix for sampling of descriptors
    binary_matrix_descriptors=zeros(num_bms,n);
    for k=1:n
        column=[ones(round(weight_descriptors(k)*num_bms),1);zeros(num_bms-round(weight_descriptors(k)*num_bms),1)];
        column=column(randperm(num_bms));
        binary_matrix_descriptors(:,k)=column;
    end
    
     %    generate the binary matrix for sampling of samples
    binary_matrix_samples=zeros(num_bms,m);
    for k=1:m
        column=[ones(round(weight_samples(k)*num_bms),1);zeros(num_bms-round(weight_samples(k)*num_bms),1)];
        column=column(randperm(num_bms));
        binary_matrix_samples(:,k)=column;
    end
    
    %     if a row is full of zeros, set them to ones
    in = sum(binary_matrix_descriptors,2)==0; % check the rows with all zeros
    binary_matrix_descriptors(in,:)=1; % change the elements into 1
    
        %     if a row is full of zeros, set them to ones
    in = sum(binary_matrix_samples,2)==0; % check the rows with all zeros
    binary_matrix_samples(in,:)=1; % change the elements into 1
    
    %     caculate the RMSECV for all sub-models
    RMSECV=zeros(1,num_bms);
    for j=1:num_bms
    CV=plscvfold(Xcal(binary_matrix_samples(j,:)==1,binary_matrix_descriptors(j,:)==1),ycal(binary_matrix_samples(j,:)==1,:),nLV_max,fold,method,0,0);
    RMSECV(j)=CV.RMSECV;
    end
    
    %     obtain new weights for descriptor sampling
    [RMSECV_sort index]=sort(RMSECV);
    meanRMSECV(i)=mean(RMSECV_sort(1:num_selected_model));
    weight_descriptors=sum(binary_matrix_descriptors(index(1:num_selected_model),:))/num_selected_model;
    %     obtain new weights for sample sampling
    weight_samples=sqrt(sum(binary_matrix_samples(index(1:num_selected_model),:))/num_selected_model);
    
    if flag==1
    figure(1)
    bar(weight)
    xlabel('varaible')
    ylabel('weight')
    axis([0 n 0 1])
    drawnow
    end
    
    W_de(:,i+1)=weight_descriptors';
    W_sa(:,i+1)=weight_samples';
      
    if i>1 && meanRMSECV(i)>=meanRMSECV(i-1)
        break
    end
 
    fprintf('The %dth iteration finished.\n',i);
end
iteration=i-1;

if i<200
meanRMSECV(i:200)=[];
W_de(:,i+1:201)=[];
W_sa(:,i+1:201)=[];
end


%% Obtain the sequence for selected variables

weight1_de=W_de==1;% informative descriptors 
diff_weight_de=zeros(n,iteration);
for k=1:iteration
    diff_weight_de(:,k)=weight1_de(:,k+1)-weight1_de(:,k);
end
informative_variables=[]; 
for j=1:iteration
    selected_new=find(diff_weight_de(:,j)==1);
    if length(selected_new)==1
        informative_variables=[informative_variables;selected_new];
    elseif length(selected_new)>=2
        [~,priority]=sort(W_de(selected_new,j),'descend');
        informative_variables=[informative_variables;selected_new(priority)];
    end
end 

weight1_sa=W_sa==1;  % informative samples
diff_weight_sa=zeros(m,iteration);
for k=1:iteration
    diff_weight_sa(:,k)=weight1_sa(:,k+1)-weight1_sa(:,k);
end
informative_samples=[];  
for j=1:iteration
    selected_new=find(diff_weight_sa(:,j)==1);
    if length(selected_new)==1
        informative_samples=[informative_samples;selected_new];
    elseif length(selected_new)>=2
        [~,priority]=sort(W_sa(selected_new,j),'descend');
        informative_samples=[informative_samples;selected_new(priority)];
    end
end 

weight0_de=W_de==0;  % interfering descriptors
diff_weight_de=zeros(n,iteration);
for k=1:iteration
    diff_weight_de(:,k)=weight0_de(:,k+1)-weight0_de(:,k);
end
interfering_variables=[];
for j=1:iteration
    eliminated_new=find(diff_weight_de(:,j)==1);
    if length(eliminated_new)==1
        interfering_variables=[eliminated_new;interfering_variables];
    elseif length(eliminated_new)>=2
          [~,priority]=sort(W_de(eliminated_new,j),'descend');  
          interfering_variables=[eliminated_new(priority);interfering_variables];
    end
end 

weight0_sa=W_sa==0; % interfering samples
diff_weight_sa=zeros(m,iteration);
for k=1:iteration
    diff_weight_sa(:,k)=weight0_sa(:,k+1)-weight0_sa(:,k);
end
interfering_samples=[];
for j=1:iteration
    eliminated_new=find(diff_weight_sa(:,j)==1);
    if length(eliminated_new)==1
        interfering_samples=[eliminated_new;interfering_samples];
    elseif length(eliminated_new)>=2
          [~,priority]=sort(W_sa(eliminated_new,j),'descend');  
          interfering_samples=[eliminated_new(priority);interfering_samples];
    end
end 

v=zeros(n,1); %uninformative descriptors
v(informative_variables)=1;
v(interfering_variables)=1;

uninformative_variables=find(v==0);
[~,priority]=sort(W_de(uninformative_variables,iteration),'descend'); 

variables_in_sequence=[informative_variables;uninformative_variables(priority);interfering_variables];

v=zeros(m,1);  %uninformative samples
v(informative_samples)=1;
v(interfering_samples)=1;

uninformative_samples=find(v==0);
[~,priority]=sort(W_sa(uninformative_samples,iteration),'descend'); 

samples_in_sequence=[informative_samples;uninformative_samples(priority);interfering_samples];
[kk,~] = size([informative_samples;uninformative_samples(priority)])

% The final filtering
cv=ones(m,n)*10;

for j=kk:m
    for i=1:n
        CV=plscvfold(Xcal(samples_in_sequence(1:j),variables_in_sequence(1:i)),ycal(samples_in_sequence(1:j)),nLV_max,fold,method,0,0);
        cv(j,i)=CV.RMSECV;
    end
end

[nSAM,nVAR]=find(cv==min(min(cv)));
RMSECV=min(min(cv));

    
    if flag==1
    figure(2)
    plot(cv)
    xlabel('varaible index')
    ylabel('RMSECV')
    end

% 
retained_variables=variables_in_sequence(1:nVAR);
variable_index=zeros(n,1);
for i=1:n
    variable_index(retained_variables)=1;
end

retained_samples=samples_in_sequence(1:nSAM);
sample_index=zeros(m,1);
for i=1:m
    sample_index(retained_samples)=1;
end

MASS.iteration=iteration;
MASS.minRMSECV=RMSECV; % the minimun RMSECV
MASS.nVAR=nVAR; % the number of selected variables
MASS.nSAM=nSAM;
MASS.retained_variables=retained_variables; % the retained variables in sequence
MASS.retained_samples=retained_samples;
MASS.variable_index=variable_index; % the index of selected variables
MASS.sample_index=sample_index;
MASS.variable_in_sequence=variables_in_sequence; % all variables in sequence
MASS.sample_in_sequence=samples_in_sequence; 
MASS.meanRMSECV=meanRMSECV; % the mean of RMSECV in each step
MASS.weight_descriptors=W_de; % the weights for variables
MASS.weight_samples=W_sa;
% MASS.time=toc; 